class checkJSON(object):
    def getKeys(self, data):
        keysAll_list = []

        def getkeys(data):  # 遍历json所有key
            if (type(data) == type({})):
                keys = data.keys()
                for key in keys:
                    value = data.get(key)
                    if (type(value) != type({}) and type(value) != type([])):
                        keysAll_list.append(key)
                    elif (type(value) == type({})):
                        keysAll_list.append(key)
                        getkeys(value)
                    elif (type(value) == type([])):
                        keysAll_list.append(key)

                for para in value:
                    if (type(para) == type({}) or type(para) == type([])):
                        getkeys(para)
                    else:
                        keysAll_list.append(para)

        getkeys(data)
        return keysAll_list

    def isExtend(self, data, tagkey):  # 检测目标字段tagkey是否在data(json数据)中
        if (type(data) != type({})):
            print('please input a json!')
        else:
            key_list = self.getKeys(data)
            for key in key_list:
                if (key == tagkey):
                    return True
        return False
    def isExistKey(self, data, keyname):
        if (keyname in data):
            return True
        else:
            return False



from PIL import Image


class photoUtil:
    target_width = 295
    target_height = 413

    def __init__(self, target_width, target_height):
        self.target_width = target_width
        self.target_height = target_height

    def trans(self, source, target):
        # 打开原始图片
        image = Image.open(source)
        # # 定义目标尺寸
        # target_width = 800
        # target_height = 600
        # 调整图片尺寸
        resized_image = image.resize((self.target_width, self.target_height))
        if str(source).upper().endswith(".PNG"):
            # 调整图片格式
            resized_image_rgb = resized_image.convert("RGB")
            resized_image_rgb.save(target)
        else:
            try:
                # 保存调整后的图片
                resized_image.save(target)
            except OSError:
                print("Failed to open file:" + source)
                resized_image_rgb2 = resized_image.convert("RGB")
                resized_image_rgb2.save(target)
                print("success to tran file:" + source)
            except  Exception as e:
                print("error:source:" + source + ",target:" + target + ",error:" + e)

    def png2jpg(self, source, target):
        image = Image.open(source)
        rgb_image = image.convert("RGB")
        rgb_image.save(target)



from dataclasses import dataclass
from datetime import datetime
from spider.BaseSpider import BaseSpider

@dataclass
class SpiderCar(BaseSpider):
    title: str #小车名称
    style: str #小车类型
    desc: str #书描述
    price: str #车价
    pic: str #车图片
    regdate: str #注册日期
    distance: str #行驶路程


    def __init__(self, username, collector, rawurl, rawdata):
        self.username = username  #采集者学号
        self.collector= collector #采集者姓名
        self.topic = "car_data"  #采集题材
        self.rawurl= rawurl #采集的原始url地址
        self.rawdata = rawdata #采集的原始数据，如记录行的json内容
        self.coll_time = datetime.now() #采集时间，实体初始方法自动填充



import json

import pandas as pd
import pymysql

from bgutils.redisUtil import redisUtil

db_config = {
    "host": "home.hddly.cn",
    "port": 53306,
    "user": "test",
    "password": "test",
    "db": "test",
    "charset": "utf8mb4"
}
class mysqlUtil:
    __redis = redisUtil()
    __connection= None

    def __init__(self):
        return

    def OpenDB(self):
        if not self.__connection:
            self.__connection = pymysql.connect(**db_config)
        return self.__connection



    # 查询上传文件是否有重复
    # def chk_file_exist(self, fdesc, fsize, duration):
    #     if fsize and duration:
    #         # 判断float相等： abs(duration-123.31)<0.01
    #         sql = "select id from sftp_files where fsize={} and  abs(duration-{})<0.01 limit 1".format(fsize, duration)
    #         connection = self.engine.connect()
    #         df = pd.read_sql(sql, con=connection)
    #         results = json.loads(df.to_json(orient='records'))
    #         if results and len(results) == 1:
    #             # 记录采集重复信息
    #             sys.stdout.write("repeat:" + fdesc + "," + str(fsize) + "," + str(duration) + "\n")
    #             return 1
    #         else:
    #             return 0
    #     else:
    #         return -1

    # def chk_file_exist_surl(self, surl, fdesc):
    #     if surl and fdesc:
    #         surl = surl[0:499]  # 只取前499位
    #         fdesc = fdesc[0:999]
    #         # 先从redis中判断 是否存在，如果不存在，再从mysql中查找，找到的话添加到redis中
    #         if self.__redis.sismember("surl", surl):
    #             return 1
    #         if self.__redis.sismember("fdesc", fdesc):
    #             return 1
    #
    #         # 判断float相等： abs(duration-123.31)<0.01
    #         sql = "select id from sftp_files where surl='{}' or fdesc='{}' limit 1".format(surl, fdesc)
    #         connection = self.engine.connect()
    #         df = pd.read_sql(sql, con=connection)
    #         results = json.loads(df.to_json(orient='records'))
    #         if results and len(results) == 1:
    #             # 记录采集重复信息
    #             self.__redis.sadd("surl", surl)
    #             self.__redis.sadd("fdesc", fdesc)
    #             return 1
    #         else:
    #             return 0
    #     else:
    #         return -1

    # 添加上传文件记录
    def sftp_file_ins(self, filename, url, stud, fdesc, fsize, duration, ftype, pid, surl):
        try:
            surl = surl[:499]
            query = ("insert into sftp_files (filename,url,stud,fdesc,fsize,duration,ftype,pid,surl) VALUES (%s, %s, "
                     "%s, %s, %s, %s, %s, %s, %s)")
            parameters = (filename, url, stud, fdesc, fsize, duration, ftype, pid, surl)
            cursor = self.__connection.cursor()
            cursor.execute(query, parameters)
            self.__connection.commit()
            cursor.close()
        except Exception as ex:
            print("sftp_file_ins error:%s" % ex)

    # 获了上传文件记录
    def get_file_list(self, ftype):
        if ftype:
            ftype = str(ftype).lower()
            sql = 'select id,url,fdesc,fsize,duration,ftype,uptime,pid from sftp_files where ftype="{}" order by uptime desc limit 100'.format(
                ftype)
        else:
            ftype = ".mp4"
            sql = 'select id,url,fdesc,fsize,duration,ftype,uptime,pid from sftp_files where ftype="{}" order by uptime desc limit 100'.format(
                ftype)
        connection = self.OpenDB()
        df = pd.read_sql(sql, con=connection)
        results = json.loads(df.to_json(orient='records'))
        if results and len(results) == 1:
            return results[0]
        elif results and len(results) > 1:
            return results
        else:
            return None
        # 获了上传文件记录

    def get_files_bypid(self, pid):
        if pid:
            pid = str(pid).lower()
            sql = 'select id,url,fdesc,fsize,duration,ftype,uptime,pid from sftp_files where pid="{}" order by id desc limit 100'.format(
                pid)
        else:
            pid = "p023101".lower()
            sql = 'select id,url,fdesc,fsize,duration,ftype,uptime,pid from sftp_files where pid="{}" order by id desc limit 100'.format(
                pid)
        connection = self.OpenDB()
        df = pd.read_sql(sql=sql, con=connection)
        results = json.loads(df.to_json(orient='records'))
        if results and len(results) == 1:
            return results[0]
        elif results and len(results) > 1:
            return results
        else:
            return None

    # def process_item(self, item, tblname):
    #     data = pd.DataFrame(dict(item), index=[0])
    #     data.to_sql(tblname, self.engine, if_exists='append', index=False)  # 'taobao_data'
    #     return item

    def escape(self, content):
        if content and (len(content) > 0) and (len(str(content).strip()) > 0):
            content = content.replace("\\\\", "\\\\\\\\");
            content = content.replace("_", "\\\\_");
            content = content.replace("%", "\\\\%");
            content = content.replace("'", "\\\\'");

        return content;



import logging
import os
from logging.handlers import RotatingFileHandler

logging.basicConfig(level=logging.INFO)

logger = logging.getLogger(__name__)

def init_log():
    # global myapp
    # myapp = tpapp
    logger.setLevel(logging.INFO)
    LOG_PATH='./logs/'
    if not os.path.exists(LOG_PATH):
        # 如果目录不存在，使用os.makedirs创建它
        os.makedirs(LOG_PATH)
    logfile = LOG_PATH + 'app.log'
    # 文件日志配置
    file_handler = RotatingFileHandler(logfile, maxBytes=1024 * 1024 * 100, backupCount=5)
    file_handler.setLevel(logging.DEBUG)
    log_format = logging.Formatter('%(asctime)s \t %(name)s \t %(levelname)s \t %(message)s')
    file_handler.setFormatter(log_format)

    # if not current_app.debug:
    logger.addHandler(file_handler)

    # 控制台日志配置
    console_handler = logging.StreamHandler()
    console_handler.setFormatter(log_format)
    logger.addHandler(console_handler)

    # 记录一些日志
    # logger.debug('This is a debug message.')
    # logger.info('This is an info message.')
    # logger.warning('This is a warning message.')
    # logger.error('This is an error message.')
    # logger.critical('This is a critical message.')


def info(str_msg):
    logger.info(str_msg)

def error(str_msg):
    logger.error(str_msg)

def debug(str_msg):
    logger.debug(str_msg)

def warning(str_msg):
    logger.warning(str_msg)


from dataclasses import dataclass
from datetime import datetime

from spider.BaseSpider import BaseSpider

@dataclass
class SpiderMeishi(BaseSpider):
    uid: str #商品ID
    author: str #作者
    title: str #商品标题
    mainingredient: str #原材料
    dateline :str #日期
    subject:str #主题
    url: str #商品url
    pic: str #商品的图片url,对应fcover

    def __init__(self, username, collector, rawurl, rawdata):
        self.username = username #采集者学号
        self.collector= collector #采集者姓名
        self.topic = "meishi_data" #采集题材
        self.rawurl= rawurl  #采集的原始url地址
        self.rawdata = rawdata #采集的原始数据，如记录行的json内容
        self.coll_time = datetime.now() #采集时间，实体初始方法自动填充



import chardet
import requests
from bs4 import BeautifulSoup

from entity.entUrlResult import entUrlResult


def getUrlContent(url):
    ua = {'User-Agent': 'Mozilla/5.0 (Windows NT 6.1; Win64; x64) Chrome/65.0.3325.181'}
    rqg = requests.get(url, headers=ua)
    rqg.encoding = chardet.detect(rqg.content)['encoding']
    # 初始化HTML
    print('rqg.encoding:',rqg.encoding)
    if rqg.encoding=='GB2312':
        html = rqg.content.decode('gbk')
    else:
        html = rqg.content.decode(rqg.encoding)
    print("格式化前文本：", html)
    return html

def getUrlResult(url):
    html = getUrlContent(url)
    soup = BeautifulSoup(html, "lxml")  # 生成BeautifulSoup对象
    tags_a = soup.find_all('a')
    if soup.find('title'):
        title = soup.find('title').text
    else:
        return
    urllist=[]
    print("所有名称为a的标签的个数:", len(tags_a))  # 获取所有名称为a的标签的个数
    for tag in tags_a:
        if tag.attrs.get('href') and (tag.attrs.get('href').startswith('http://')
                                      or tag.attrs.get('href').startswith('https://')):
            print(tag.attrs.get('href'))
            urllist.append(tag.attrs.get('href'))
    rlt = entUrlResult(url,title,html,urllist)
    return rlt

def getUrlLinkDeeps_0(url):
    urllist_all = []
    urllist_0 = []
    urllist_0.extend(getUrlResult(url).urls)
    for url0 in urllist_0:
        print("url0:",url0)
    urllist_all.extend(urllist_0)
    return urllist_all

def getUrlLinkDeeps_1(url):
    urllist_all = []
    urllist_0 = []
    urllist_1 = []
    urllist_0.extend(getUrlResult(url).urls)
    for url0 in urllist_0:
        print("url0:",url0)
        urllist_1.extend(getUrlResult(url0).urls)
    urllist_all.extend(urllist_0)
    urllist_all.extend(urllist_1)
    return urllist_all




import logging
from dataclasses import dataclass
from enum import Enum
from flask import json
from flask.json import dumps
from kafka import KafkaProducer, KafkaConsumer

# 非阻塞取消息的间隔（秒）
KAFKA_POLL_INTERVAL = 0.5
def init_kafak():
    global KAFKA_BOOTSTRAP_SERVERS
    KAFKA_BOOTSTRAP_SERVERS = 'home.hddly.cn:9092'
    global KAFKA_TOPIC_NAME  # Kafka主题
    KAFKA_TOPIC_NAME = 'tp_spider'
    global KAFKA_GROUP_ID
    KAFKA_GROUP_ID = 'tp_spider'
    global producer_spider
    producer_spider = KafkaProducer(
        bootstrap_servers=KAFKA_BOOTSTRAP_SERVERS,
        value_serializer=lambda m: json.dumps(m).encode('utf-8')
    )
    global consumer_spider
    consumer_spider = KafkaConsumer(
        KAFKA_TOPIC_NAME,
        bootstrap_servers=KAFKA_BOOTSTRAP_SERVERS,
        auto_offset_reset='earliest',  # 从最早的消息开始消费
        enable_auto_commit=True,  # 自动提交偏移量
        group_id=KAFKA_GROUP_ID,  # 消费者组ID，用于协调消费和偏移量管理
        value_deserializer=lambda m: json.loads(m.decode('utf-8'))  # 假设消息是JSON格式
    )

init_kafak()
@dataclass
class EnumUserAction(Enum):
    USER_MENU_CLICK = "user_menu_click"  # 用户登陆的目录
    USER_STUDY_TIME = "user_study_time"  # 学生学习时长


@dataclass
class userData():
    userid: str
    useract: str
    data: str

    def __init__(self, userid, useract, data):
        self.userid = userid
        self.useract = getCommKey(useract)
        self.data = data


def getCommKey(keyname):
    try:
        key = keyname.value
        return key
    except Exception as ex:
        logging.error("error:getcommkey" + str(ex))


def sendKafkaUserAction(current_user, datastrin):
    datastr = userData(str(current_user.user_id), EnumUserAction.USER_MENU_CLICK, datastrin)  # 获取 POST 请求中的 JSON 数据
    return sendKafkaMsg(datastr)


def sendKafkaSpiderData(spiderdata):
    return sendKafkaMsg(spiderdata)


def sendKafkaMsg(datastr):
    # message = json.dumps(datastr).encode('utf-8')  # 将数据转换为 JSON 字符串并编码为字节
    message = json.dumps(datastr) #改在producer_spider中编码
    # 发送消息到 Kafka
    producer_spider.send(KAFKA_TOPIC_NAME, value=message)
    return 1


def getKafkaMsg(current_user):
    # 消费消息
    datas = []
    try:
        # 非阻塞方式
        messages = consumer_spider.poll(timeout_ms=int(KAFKA_POLL_INTERVAL * 1000))
        if messages:
            for topic_part, message_list in messages.items():
                for message in message_list:
                    # 处理消息
                    logging.info(f"Partition: {message.partition}, Offset: {message.offset}, Value: {message.value}")
                    data = message.value
                    datas.append(data)
        # 阻塞方式
        # for message in consumer:
        #     data=message.value
        #     # message是一个ConsumerRecord对象，包含消息的元数据（如偏移量、主题、分区等）和值
        #     print(f"Partition: {message.partition}, Offset: {message.offset}, Value: {message.value}")
        #     datas.append(data)
        return datas
    except Exception as ex:
        logging.error("error:get_message:" + str(ex))
        return None


def close():
    consumer_spider.close()



class entUrlResult(object):
    rlt=dict(key=0,value=None)
    rlt = {"url": '', "title": '', "html":'' , "urllist":''}

    def __new__(self, url, title,html,urllist):
        # url,title,html,urllist
        self.rlt["url"] = url
        self.rlt["title"] = title
        self.rlt["html"] = html
        self.rlt["urllist"] = urllist
        return self.rlt


import socket
def check_ip(host,port,timeout=5):
    try:
        # 创建一个socket对象
        sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        # 设置超时时间
        sock.settimeout(timeout)
        # 尝试连接到服务器
        sock.connect((host, port))
        # 如果连接成功，关闭socket并返回True
        sock.close()
        return True
    except socket.error as e:
        # 如果连接失败，打印异常并返回False
        print(f"连接到 {host}:{port} 失败: {e}")
        return False


from dataclasses import dataclass
from datetime import datetime
from spider.BaseSpider import BaseSpider

@dataclass
class SpiderNews(BaseSpider):
    newsTitle: str #新闻标题
    newsUrl: str #新闻地址url
    ctime: str  #新闻发布时间
    media_name: str #发布媒体
    keywords: str #关键词

    def __init__(self, username, collector, rawurl, rawdata):
        self.username = username #采集者学号
        self.collector= collector #采集者姓名
        self.topic = "news_data" #采集题材
        self.rawurl= rawurl #采集的原始url地址
        self.rawdata = rawdata #采集的原始数据，如记录行的json内容
        self.coll_time = datetime.now() #采集时间，实体初始方法自动填充



from dataclasses import dataclass
from datetime import datetime

from spider.BaseSpider import BaseSpider


@dataclass
class SpiderMusic(BaseSpider):
    id: str  #ID,如"222666"
    song_url: str #歌曲url
    song_name: str  #歌曲名
    artist_url: str #歌手url
    artist_name: str #歌手名
    album_name: str #专辑名
    album_url: str #专辑url

    def __init__(self, username, collector, rawurl, rawdata):
        self.username = username #采集者学号
        self.collector= collector #采集者姓名
        self.topic = "music_data_1" #采集题材
        self.rawurl= rawurl #采集的原始url地址
        self.rawdata = rawdata #采集的原始数据，如记录行的json内容
        self.coll_time = datetime.now() #采集时间，实体初始方法自动填充




import pymongo
from pymongo import MongoClient

class mongoUtil:
    __mongo_ip="home.hddly.cn"
    __port="57017"
    __urlendstr="/?readPreference=primary&appname=MongoDB%20Compass&directConnection=true&ssl=false"

    def OpenDB(self, dbname):
        self.__mongo_ip = "home.hddly.cn"
        self.__port = "57017"
        uri = "mongodb://"+self.__mongo_ip+":" + self.__port + self.__urlendstr
        self.con = MongoClient(uri, connect=False)
        self.db = self.con[dbname]

    def OpenDBYgxy(self, dbname):
        self.__mongo_ip = "10.255.10.52"
        self.__port = "27017"
        uri = "mongodb://" + self.__mongo_ip + ":" + self.__port + self.__urlendstr
        self.con = MongoClient(uri, connect=False)
        self.db = self.con[dbname]

    def OpenDBAny(self, ip,port,dbname):
        self.__mongo_ip = ip
        self.__port = port
        uri = "mongodb://" + self.__mongo_ip + ":" + self.__port + self.__urlendstr
        self.con = MongoClient(uri, connect=False)
        self.db = self.con[dbname]

    def OpenDBUri(self, dburi,dbname):
        self.con = MongoClient(dburi, connect=False)
        self.db = self.con[dbname]

    def closeDB(self):
        self.con.close()

    def process_item(self, item, collection):
        jsontmp = dict(item)
        self.collection = self.db[collection]
        self.collection.insert_one(jsontmp)
        return item

    def process_items(self, items, collection):
        docs = []
        for item in items:
            jsontmp = dict(item)
            docs.append(jsontmp)
        if len(docs) > 0:
            self.collection = self.db[collection]
            self.collection.insert_many(docs)
        return items

    def get_db_count(self, collection, collector):
        self.collection = self.db[collection]
        rowcount = self.collection.count_documents({'collector': collector})
        return {'collector': collector, 'collcount': rowcount}

    def get_db_count_all(self, collection):
        self.collection = self.db[collection]
        cur = self.collection.aggregate([
            {"$group": {"_id": "$collector", "count": {"$sum": 1}}},
            {"$sort": {"count": -1}}
        ])
        # $group操作符去利用一个指定的键进行分组
        # $collector - key
        # $sum累加器进行文档的统计计算
        # $sort 排序
        counts = dict()
        for document in cur:
            # 'ascii' codec can't encode characters in position 0-2: ordinal not in range(128) ,
            # 。你可以使用Python的type()函数来检查数据类型，以及使用.encode('utf-8')和.decode('utf-8')来显式转换编码
            try:
                if document['_id'] and document['count']:
                    collecter = document['_id']
                    counts[collecter] = document['count']
                # print(document['_id'] + '\t' + str(document['count']))
            except Exception as ex:
                print("get_db_count_all" + str(ex) + ",document:" + str(document))
        return counts

    def set_db_index(self):
        self.db['users'].create_index([('id', pymongo.ASCENDING)])
        self.db['weibos'].create_index([('id', pymongo.ASCENDING)])

    def insertMongdbOne(self,collection, jsoninfo):
        self.collection = self.db[collection]
        print(jsoninfo)
        self.collection.insert_one(jsoninfo)

if __name__ == '__main__':

    mongo = mongoUtil()
    mongo.OpenDB("pythondb")

    mongo.closeDB()
    exit()



from dataclasses import dataclass
from datetime import datetime

from spider.BaseSpider import BaseSpider

@dataclass
class SpiderJobs(BaseSpider):
    job_name: str #职位名称，如"大数据开发"
    job_Id: str #职位ID,如"143759451"
    job_url: str #职位链接，如"https://jobs.51job.com/qingdao/143759451.html?s=sou_sou_soulb&t=0_0&re…"
    company_name :str #招聘企业，如"青岛某科技信息有限公司"
    release_time :str #职位发布晶时间，如"2023-05-22 09:26:08"
    job_salary: str #薪资 "7千-1.4万"
    job_address: str #工作地点,如"青岛"
    work_year: str #工作年限要求,如    "无需经验"
    degree_request:str #工作学历要求，如 "本科"
    job_explain: str #经验要求，如"无需经验,hadoop,java,hive,数据库,数据分析,…"

    def __init__(self, username, collector, rawurl, rawdata):
        self.username = username #采集者学号
        self.collector= collector #采集者姓名
        self.topic = "jobs_data" #采集题材
        self.rawurl= rawurl #采集的原始url地址
        self.rawdata = rawdata #采集的原始数据，如记录行的json内容
        self.coll_time = datetime.now() #采集时间，实体初始方法自动填充



from dataclasses import dataclass
from datetime import datetime

from spider.BaseSpider import BaseSpider

@dataclass
class SpiderVideo(BaseSpider):
    id: str  #视频ID,如"222666"
    typename: str #视频分类
    title: str  #视频标题,如"生死狙击之僵尸前线"
    playlink: str #视频播放地址
    pic: str #视频图片地址
    duration: str #视频播放时长
    introduce: str #视频介绍，如 "全新变异体-月兔精上线"


    def __init__(self, username, collector, rawurl, rawdata):
        self.username = username #采集者学号
        self.collector= collector #采集者姓名
        self.topic = "videos_data" #采集题材
        self.rawurl= rawurl #采集的原始url地址
        self.rawdata = rawdata #采集的原始数据，如记录行的json内容
        self.coll_time = datetime.now() #采集时间，实体初始方法自动填充
